;; Generated package description from rainbow-mode.el  -*- no-byte-compile: t; lexical-binding:t -*-
(define-package "rainbow-mode" "1.0.7.0.20260905.0" "Colorize color names in buffers" 'nil :commit "9d333d3a92132c2a9057d2a39e1ebd8cc575a1bb" :url "https://elpa.gnu.org/packages/rainbow-mode.html" :authors '(("Julien Danjou" . "julien@danjou.info")) :maintainer '("Julien Danjou" . "julien@danjou.info") :keywords '("faces"))
